package atomfeed;

import java.io.*;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import javax.ws.rs.*;
import javax.ws.rs.core.*;
import javax.ws.rs.ext.*;
import org.apache.abdera.Abdera;
import org.apache.abdera.model.*;
import org.apache.abdera.writer.Writer;

/**
 * JAX-RS-Provider-Klasse, damit JAX-RS die Klassen Feed und Entry lesen und schreiben kann.<br>
 * Siehe auch:<br>
 * org.apache.wink.providers.abdera.AbderaAtomFeedProvider und AbderaAtomEntryProvider
 * sowie<br>
 * org.jboss.resteasy.plugins.providers.atom.AbderaFeedProvider und AbderaEntryProvider
 */
@Provider
@Produces( MediaType.APPLICATION_ATOM_XML )
@Consumes( MediaType.APPLICATION_ATOM_XML )
public class AbderaJaxRsProvider implements MessageBodyReader<Object>, MessageBodyWriter<Object>
{
   // Die Abdera-Klasse ist threadsafe:
   private final static Abdera abdera = new Abdera();

   public static Abdera getAbdera()
   {
      return abdera;
   }

   @Override
   public long getSize( Object obj, Class<?> clss, Type type, Annotation[] annot, MediaType mediaType )
   {
      return -1;
   }

   @Override
   public boolean isReadable( Class<?> clss, Type type, Annotation[] annot, MediaType mediaType )
   {
      return (Feed.class.isAssignableFrom( clss ) || Entry.class.isAssignableFrom( clss ));
   }

   @Override
   public boolean isWriteable( Class<?> clss, Type type, Annotation[] annot, MediaType mediaType )
   {
      return (Feed.class.isAssignableFrom( clss ) || Entry.class.isAssignableFrom( clss ));
   }

   @Override
   public Object readFrom( Class<Object> feedOrEntry, Type type, Annotation[] annot, MediaType mediaType,
         MultivaluedMap<String, String> headers, InputStream inputStream ) throws IOException
   {
      Document<Element> doc = getAbdera().getParser().parse( inputStream );
      Element el = doc.getRoot();
      if( feedOrEntry.isAssignableFrom( el.getClass() ) ) {
         return el;
      }
      throw new IOException( "Falsche Klasse '" + el.getClass().getName() + "' statt '" + feedOrEntry.getName() + "'." );
   }

   @Override
   public void writeTo( Object feedOrEntry, Class<?> clss, Type type, Annotation[] annot, MediaType mediaType,
         MultivaluedMap<String, Object> headers, OutputStream outputStream ) throws IOException, WebApplicationException
   {
      Writer writer = getAbdera().getWriterFactory().getWriter( "PrettyXML" );
      writer.writeTo( ( feedOrEntry instanceof Feed ) ? (Feed) feedOrEntry : (Entry) feedOrEntry, outputStream );
   }
}
