package atomfeed;

import java.io.*;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import javax.xml.bind.*;
import org.apache.abdera.model.*;

/**
 * Simple Simulation eines Datenspeichers.
 * In der Realitaet koennten die Daten in einer Datenbank gespeichert werden.
 */
public class JaxbDatenDAO
{
   public static final String HTTP_201_CREATED = "201 Created";

   // Map als Datenbank-Simulation:
   private static Map<Long,MeineDaten> daten = new ConcurrentHashMap<Long,MeineDaten>();

   public static void fillFeedData( Feed feed, String uri )
   {
      feed.setId(       uri );
      feed.setUpdated(  new Date() );
      feed.setTitle(    "Mein Feed-Titel" );
      feed.setSubtitle( "Mein Feed-Untertitel" );
      feed.addAuthor(   "ich" );
      feed.addLink(     "http://www.torsten-horn.de/techdocs/jee-rest.htm" );
      feed.addLink(     uri, "self" );
   }

   public static void fillEntryData( Entry entry, Long idNum, String uri ) throws JAXBException
   {
      Long id = ( idNum != null ) ? idNum : Long.valueOf( 0 );
      entry.setId(      uri + "/entry/" + id );
      entry.setUpdated( new Date() );
      entry.setTitle(   "Titel zum Entry mit der ID " + id );

      MeineDaten md = daten.get( id );
      if( md != null ) {
         JAXBContext jaxbCtx = JAXBContext.newInstance( MeineDaten.class );
         StringWriter writer = new StringWriter();
         jaxbCtx.createMarshaller().marshal( md, writer );
         entry.setContent( writer.toString(), "application/vnd.meinmediatyp+xml" );
      }
   }

   public static String storeContent( Entry entry ) throws JAXBException
   {
      JAXBContext jaxbCtx = JAXBContext.newInstance( MeineDaten.class );
      MeineDaten  md = (MeineDaten) jaxbCtx.createUnmarshaller().unmarshal( new StringReader( entry.getContent() ) );
      daten.put( new Long( md.i ), md );
      return HTTP_201_CREATED;
   }

   public static Set<Long> getListOfIDs()
   {
      return daten.keySet();
   }
}
