package atomfeed;

import java.io.*;
import java.net.*;
import java.net.URL;
import java.util.Iterator;
import org.apache.abdera.Abdera;
import org.apache.abdera.model.*;
import org.apache.abdera.parser.Parser;
import org.apache.abdera.writer.Writer;

public class AtomFeedPrint
{
   /**
    * Anzeige eines Atomfeeds: wahlweise nur alle Titel oder vollstaendig (z.B. als XML).<br>
    * @param args Als ersten Kommandozeilenparameter die URL zum Atomfeed uebergeben, z.B.:<br>
    *    {@link "http://localhost:4434/atomfeed"}<br>
    *    {@link "http://news.google.com/news?ned=us&topic=h&output=atom"}<br>
    *    {@link "http://www.heise.de/newsticker/heise-atom.xml"}<br>
    *    Der zweite Parameter kann leer sein ("") oder "PrettyXML"<br>
    *    Der dritte Parameter kann leer sein ("") oder "alles" (fuer Titel + Deskription)<br>
    *    Falls Proxy: Als vierten und fuenften Parameter Proxy-Host und -Port uebergeben, z.B. 127.0.0.1 3128
    * Verwendet: {@link "http://abdera.apache.org/docs/api/org/apache/abdera/Abdera.html"}<br>
    */
   @SuppressWarnings("resource")
   public static void main( String[] args ) throws IOException
   {
      String atomfeedUrl = ( args.length > 0 ) ? args[0] : "http://localhost:4434/atomfeed";
      String prettyxml   = ( args.length > 1 ) ? args[1] : null; // z.B. PrettyXML
      String alles       = ( args.length > 2 ) ? args[2] : null; // z.B. alles
      String proxyHost   = ( args.length > 3 ) ? args[3] : null; // z.B. 127.0.0.1
      String proxyPort   = ( args.length > 4 ) ? args[4] : null; // z.B. 3128
      boolean all = alles != null && alles.trim().toLowerCase().startsWith( "all" );

      Abdera      abdera = new Abdera();
      Parser      parser = abdera.getParser();
      URL         url = new URL( atomfeedUrl );
      InputStream inp = null;

      if( proxyHost != null && proxyPort != null ) {
         Proxy proxy = new Proxy( Proxy.Type.HTTP, new InetSocketAddress( proxyHost, Integer.parseInt( proxyPort ) ) );
         inp = url.openConnection( proxy ).getInputStream();
      } else {
         inp = url.openStream();
      }

      try {
         Document<Element> elemDoc  = parser.parse( inp, url.toString() );
         Element elem = elemDoc.getRoot();

         if( elem instanceof Feed ) {
            Feed feed = (Feed) elem;
            System.out.println( "\n\n---- Nur Titel ----\n" );
            System.out.println( feed.getTitle() );
            for( Entry entry : feed.getEntries() ) {
               System.out.println( "\t" + entry.getTitle() );
            }
            if( all ) {
               System.out.println( "\n\n---- Vollstaendig ----\n" );
               if( prettyxml != null && prettyxml.trim().length() == 0 ) { prettyxml = null; }
               Writer writer = abdera.getWriterFactory().getWriter( prettyxml );
               writer.writeTo( feed, System.out );
            }
         } else {
            int i0, i1, i2;
            Iterator<Element> it0, it1, it2;
            for( i0 = 0, it0 = elem.iterator(); it0.hasNext() && i0++ < 50; ) {
               for( i1 = 0, it1 = printNextElement( it0, all, "" ); it1 != null && it1.hasNext() && i1++ < 100; ) {
                  for( i2 = 0, it2 = printNextElement( it1, all, "" ); it2 != null && it2.hasNext() && i2++ < 50; ) {
                     printNextElement( it2, all, "    " );
                  }
               }
            }
         }
      } finally {
         if( inp != null ) { inp.close(); }
      }
   }

   private static Iterator<Element> printNextElement( Iterator<Element> it, boolean all, String prefix )
   {
      if( it == null ) { return null; }
      Element el = it.next();
      if(              "title".equals( el.getQName().toString() ) ) { System.out.println( prefix + el.getText() ); }
      if( all && "description".equals( el.getQName().toString() ) ) { System.out.println( prefix + el.getText() + "\n" ); }
      return el.iterator();
   }
}
