package atomfeed;

import java.io.IOException;
import java.net.URL;
import javax.ws.rs.core.MediaType;
import com.sun.jersey.api.client.*;
import com.sun.jersey.api.container.grizzly2.GrizzlyServerFactory;
import org.apache.abdera.Abdera;
import org.apache.abdera.model.*;
import org.apache.abdera.parser.ParseException;
import org.glassfish.grizzly.http.server.HttpServer;
import org.junit.*;

public class JaxbAtomFeedServiceTest
{
   static final Abdera abdera = new Abdera();

   @Test
   public void testJaxbAtomFeedService() throws IllegalArgumentException, IOException
   {
      final String urlSrvr  = "http://localhost:4434";
      final String urlFeed  = urlSrvr + "/atomfeedjaxb";
      final String urlEntry = urlSrvr + "/atomfeedjaxb/entry";
      final long   idNum13  = 13;
      final long   idNum42  = 42;

      // Testserver:
      HttpServer srv = GrizzlyServerFactory.createHttpServer( urlSrvr );

      try {

         // Schreibe zwei Entries:
         Assert.assertEquals( JaxbDatenDAO.HTTP_201_CREATED, writeEntry( urlEntry, idNum13 ) );
         Assert.assertEquals( JaxbDatenDAO.HTTP_201_CREATED, writeEntry( urlEntry, idNum42 ) );

         // Lies und teste beide Entries:
         readAndCheckEntry( urlEntry, idNum13 );
         readAndCheckEntry( urlEntry, idNum42 );

         // Teste Feed:
         URL            urlF = new URL( urlFeed );
         Document<Feed> docF = abdera.getParser().parse( urlF.openStream(), urlF.toString() );
         Feed           feed = docF.getRoot();
         Assert.assertEquals( "Mein Feed-Titel", feed.getTitle() );
         Assert.assertEquals( "ich", feed.getAuthor().getName() );
         Assert.assertEquals( 2, feed.getEntries().size() );
         for( Entry entry : feed.getEntries() ) {
            Assert.assertTrue( entry.getTitle().startsWith( "Titel zum Entry mit der ID " ) );
            Assert.assertTrue( entry.getContent().startsWith( "<ns2:meineDaten xmlns:ns2=\"de.meinefirma.meinnamespace\">" ) );
         }

      } finally {
         // Testserver beenden:
         srv.stop();
      }
   }

   private static String writeEntry( String urlEntry, long idNum )
   {
      WebResource wrs = Client.create().resource( urlEntry );
      return wrs.type( MediaType.APPLICATION_ATOM_XML + "; charset=utf-8" )
              .accept( MediaType.APPLICATION_ATOM_XML + "; charset=utf-8" ).put( String.class,
                    "<entry xmlns='http://www.w3.org/2005/Atom'>" +
                    "<id>http://localhost:4434/atomfeedjaxb/entry/" + idNum + "</id>" +
                    "<updated>2011-01-02T00:00:00.000Z</updated>" +
                    "<title type='text'>Titel zum Entry mit der ID " + idNum + "</title>" +
                    "<content type='application/vnd.meinmediatyp+xml'>" +
                    "<ns2:meineDaten xmlns:ns2='de.meinefirma.meinnamespace'>" +
                    "<i xmlns=''>" + idNum + "</i>" +
                    "<s xmlns=''>Mein JAXB-Daten-Text " + idNum + "</s>" +
                    "</ns2:meineDaten></content></entry>" );
   }

   private static Entry readAndCheckEntry( String urlEntry, long idNum ) throws ParseException, IOException
   {
      URL url = new URL( urlEntry + "/" + idNum );
      Entry entry = (Entry) abdera.getParser().parse( url.openStream(), url.toString() ).getRoot();
      Assert.assertEquals( "Titel zum Entry mit der ID " + idNum, entry.getTitle() );
      Assert.assertEquals( "<ns2:meineDaten xmlns:ns2=\"de.meinefirma.meinnamespace\">\n" +
                           "      <i>" + idNum + "</i>\n" +
                           "      <s>Mein JAXB-Daten-Text " + idNum + "</s>\n" +
                           "    </ns2:meineDaten>", entry.getContent() );
      return entry;
   }
}
