package contractfirstservice;

import java.io.File;
import java.net.URI;
import javax.xml.bind.JAXBElement;
import org.glassfish.api.deployment.DeployCommandParameters;
import org.glassfish.api.embedded.*;
import org.junit.*;
import com.sun.jersey.api.client.*;
import contractfirstgenerated.*;

public class ContractfirstServiceTest
{
   private Server           glassfish;
   private EmbeddedDeployer deployer;
   private WebResource      wrs; // (WebResource ist thread safe)

   @Before
   public void setUp() throws Exception
   {
      // Embedded GlassFish Server:
      final URI    BASE_URI = new URI( "http://localhost:4434/JaxRsEmbeddedGlassFish/rest" );
      final int    PORT     = 4434;
      final String NAME     = "JaxRsEmbeddedGlassFish";
      glassfish = (new Server.Builder( NAME )).build();
      glassfish.addContainer( glassfish.createConfig( ContainerBuilder.Type.web ) );
      glassfish.createPort( PORT );
      glassfish.start();

      // Deploy WAR:
      ScatteredArchive.Builder war = new ScatteredArchive.Builder( NAME, new File( "target/" ) );
      war.resources(    new File( "src/main/webapp" ) );
      war.addMetadata(  new File( "src/main/webapp/WEB-INF/web.xml" ) );
      war.addClassPath( new File( "target/classes" ).toURI().toURL() );
      DeployCommandParameters params = new DeployCommandParameters();
      params.name = NAME;
      deployer = glassfish.getDeployer();
      deployer.deploy( war.buildWar(), params );

      // Client:
      wrs = Client.create().resource( BASE_URI );
   }

   @After
   public void tearDown() throws Exception
   {
      deployer.undeployAll();
      glassfish.stop();
   }

   @Test
   public void testContractfirstServiceMitEmbeddedGlassFish()
   {
      // Mit JAXB und mit bequemen Java-Objekten:
      InputTO inpTO = new InputTO();
      inpTO.setI( 42 );
      inpTO.setS( "abc xyz" );
      JAXBElement<InputTO> inpJaxb = (new ObjectFactory()).createInputTO( inpTO );
      ResultTO resTO = wrs.path( "contractfirst" ).type( "text/xml; charset=utf-8" ).accept( "text/xml; charset=utf-8" ).post(
                           ResultTO.class, inpJaxb );
      Assert.assertEquals(              84, resTO.getI() );
      Assert.assertEquals( "abc xyz - ret", resTO.getS() );

      // Ohne JAXB und mit XML-Strings (Namespace beachten):
      String resXml = wrs.path( "contractfirst" ).type( "text/xml; charset=utf-8" ).accept( "text/xml; charset=utf-8" ).post(
                          String.class, "<inputTO xmlns='mein.ns'><i>42</i><s>abc xyz</s></inputTO>" );
      Assert.assertEquals( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" +
                           "<resultTO xmlns=\"mein.ns\"><i>84</i><s>abc xyz - ret</s></resultTO>", resXml );
   }
}
