package de.meinefirma.meinprojekt.client;

import javax.ws.rs.core.MultivaluedMap;
import com.sun.jersey.api.client.*;
import com.sun.jersey.core.util.MultivaluedMapImpl;

/** RESTful-Webservice-Client */
public class BuecherRestClient
{
   public static void main( String[] args )
   {
      Client      client = Client.create();
      WebResource webResource = client.resource( "http://localhost:8080/JaxRsBuecherverwaltung/rest/Artikel/Buecher" );

      if( args.length > 1 && args[0].toLowerCase().equals( "getbuchbyid" ) ) {
         System.out.println( "\nBuch mit ID " + args[1] + ":" );
         System.out.println( getBuchById( webResource, args[1] ) );
      }
      else if( args.length > 2 && args[0].toLowerCase().equals( "findebuecher" ) ) {
         System.out.println( "\nFinde Buecher mit ISBN " + args[1] + " oder Titel '" + args[2] + "':" );
         System.out.println( findeBuecher( webResource, null, args[1], args[2] ) );
      }
      else if( args.length > 4 && args[0].toLowerCase().equals( "putbuch" ) ) {
         System.out.println( "\nAendere Daten zum Buch mit der ID " + args[1] + ":" );
         System.out.println( putBuch( webResource, args[1], "isbn=" + args[2] + "&titel=" + args[3] + "&preis=" + args[4] ) );
      }
      else if( args.length > 3 && args[0].toLowerCase().equals( "postbuch" ) ) {
         System.out.println( "\nNeues Buch anlegen:" );
         ClientResponse response = postBuch( webResource, args[1], args[2], new Double( args[3] ) );
         System.out.println( response.getStatus() + " " + response.getStatusInfo() );
         System.out.println( response.getEntity( String.class ) );
      }
      else if( args.length > 1 && args[0].toLowerCase().equals( "deletebuch" ) ) {
         System.out.println( "\nLoesche Buch mit ID " + args[1] + ":" );
         System.out.println( deleteBuch( webResource, args[1] ) );
      }
   }

   static String getBuchById( WebResource webResource, String id )
   {
      return webResource.path( id ).get( String.class );
   }

   static String findeBuecher( WebResource webResource, String id, String isbn, String titel )
   {
      MultivaluedMap<String,String> queryParams = new MultivaluedMapImpl();
      if( id    != null ) queryParams.add( "id",    id );
      if( isbn  != null ) queryParams.add( "isbn",  isbn );
      if( titel != null ) queryParams.add( "titel", titel );
      return webResource.queryParams( queryParams ).get( String.class );
   }

   static ClientResponse putBuch( WebResource webResource, String id, String data )
   {
      return webResource.path( id ).put( ClientResponse.class, data );
   }

   static ClientResponse postBuch( WebResource webResource, String isbn, String titel, Double preis )
   {
      MultivaluedMap<String,String> formData = new MultivaluedMapImpl();
      formData.add( "isbn",  isbn );
      formData.add( "titel", titel );
      formData.add( "preis", "" + preis );
      return webResource.type( "application/x-www-form-urlencoded" ).post( ClientResponse.class, formData );
   }

   static ClientResponse deleteBuch( WebResource webResource, String id )
   {
      return webResource.path( id ).delete( ClientResponse.class );
   }
}
